/**
 * This is the main cog_test program file.
 */
#include <stdio.h>
#include <propeller.h>
#include "cog_test.h"

/**
 * @brief Load a COGC driver
 * @param id The COGC driver name
 * @param param Parameter to pass to the driver
 * @returns the id of the COG that was loaded
 */
#define load_cogc_driver(id, param) \
            load_cog_driver_xmm( \
                _load_start_##id##_cog, \
                _load_stop_##id##_cog - _load_start_##id##_cog, \
                (uint32_t *)(param))
                
/* control block for cogA */
volatile struct {
    unsigned stack[STACK_SIZE_A];
    volatile Mailbox_t m;
} parA;

/* control block for cogB */
volatile struct {
    unsigned stack[STACK_SIZE_B];
    volatile Mailbox_t m;
} parB;

/* start cogA */
void startA(volatile void *parptr)
{
    extern unsigned int _load_start_cogA_cog[];
    parA.m.started = 0;
#if defined(__PROPELLER_USE_XMM__)
    extern unsigned int _load_stop_cogA_cog[];
    load_cogc_driver(cogA,(uint32_t *)parptr);
#else
    cognew(_load_start_cogA_cog, parptr);
#endif
}

/* start cogB */
void startB(volatile void *parptr)
{
    extern unsigned int _load_start_cogB_cog[];
    parB.m.started = 0;
#if defined(__PROPELLER_USE_XMM__)
    extern unsigned int _load_stop_cogB_cog[];
    load_cogc_driver(cogB, (uint32_t *)parptr);
#else
    cognew(_load_start_cogB_cog, parptr);
#endif
}

int main(void)
{
    waitcnt(CLKFREQ+CNT);  

/* start the A cog */
    printf("starting cogA\n");
    startA(&parA.m);
    while(parA.m.started == 0)
      ;
    printf("A: %s\n",parA.m.cog_message);
    parA.m.started = 0;

/* start the B cog */
    printf("starting cogB\n");
    startB(&parB.m);
    while(parB.m.started == 0)
      ;
    printf("B: %s\n",parB.m.cog_message);
    parB.m.started = 0;

    printf("Now running\n");
 
/* loop forever */
    while(1) {
        printf("A: %s %11u\n",parA.m.cog_message, parA.m.started);
        printf("B: %s %11u\n",parB.m.cog_message, parB.m.started);
        waitcnt(CLKFREQ/2+CNT);  
    }
    return 0;
}